import enum


class VIDEO_FORMAT(enum.Enum):
	RAW = 0 # raw bytes
	PIL = 1 # Pillow (PIL) Image object
	PNG = 2 # PNG file bytes
	QT5 = 3 # pyQt5

class MOUSEBUTTON(enum.Enum):
	MOUSEBUTTON_HOVER = -1
	MOUSEBUTTON_LEFT = 0
	MOUSEBUTTON_RIGHT = 1
	MOUSEBUTTON_MIDDLE = 2
	MOUSEBUTTON_WHEEL_UP = 3
	MOUSEBUTTON_WHEEL_DOWN = 4
	MOUSEBUTTON_5 = 5
	MOUSEBUTTON_6 = 6
	MOUSEBUTTON_7 = 7
	MOUSEBUTTON_8 = 8
	MOUSEBUTTON_9 = 9
	MOUSEBUTTON_10 = 10
	MOUSEBUTTON_11 = 11
	MOUSEBUTTON_12 = 12
	MOUSEBUTTON_13 = 13
	MOUSEBUTTON_14 = 14
	MOUSEBUTTON_15 = 15
	MOUSEBUTTON_16 = 16
	MOUSEBUTTON_17 = 17
	MOUSEBUTTON_18 = 18
	MOUSEBUTTON_19 = 19
	MOUSEBUTTON_20 = 20
	MOUSEBUTTON_21 = 21
	MOUSEBUTTON_22 = 22
	MOUSEBUTTON_23 = 23
	MOUSEBUTTON_24 = 24
	MOUSEBUTTON_25 = 25
	MOUSEBUTTON_26 = 26
	MOUSEBUTTON_27 = 27
	MOUSEBUTTON_28 = 28
	MOUSEBUTTON_29 = 29
	MOUSEBUTTON_30 = 30